(function ($) {

  var images = [];
  var path = '';
  var currentSlide = 0;
  var presentationLength = 0;
  var currentImageIndex = 0;
  var zeroPadding = '';
  var context = null;

  function getNewImage(src) {
    var image = new Image();
    image.src = src;
    return image;
  }

  function drawImage(image, side) {
    if (context) {
      context.clearRect(0, 0, context.canvas.width, context.canvas.height);
      context.drawImage(image, 0, 0, context.canvas.width, context.canvas.height);
    }
  }

  function nextSlide() {
    if (currentSlide < presentationLength - 1) {
      if (images[currentSlide + 1]) {
        drawImage(images[currentSlide + 1]);
        currentSlide += 1;
      }
    } else {
      if (images[0]) {
        drawImage(images[0]);
        currentSlide = 0;
      }
    }
  }

  function prevSlide() {
    if (currentSlide === 0) {
      if (images[presentationLength - 1]) {
        drawImage(images[presentationLength - 1]);
        currentSlide = presentationLength - 1;
      }
    } else {
      if (images[currentSlide - 1]) {
        drawImage(images[currentSlide - 1]);
        currentSlide -= 1;
      }
    }
  }

  function resizeCanvasToImageRatio(imageWidth, imageHeight) {
    var perc = imageWidth / imageHeight;
    context.canvas.height = Math.floor(context.canvas.width / perc);
  }

  function initPresentation(prevButton, nextButton, callback) {
    currentSlide = 0;
    images = [];
    if (path && context) {
      var image;
      for (var i = 0; i < presentationLength; i += 1) {
        var prevIndex = currentImageIndex + 1;
        if (currentImageIndex.toString().length > prevIndex.toString().length) {
          zeroPadding = zeroPadding.substr(0, zeroPadding.length - 1);
        }
        image = getNewImage(path + zeroPadding + currentImageIndex + '.svg');
        images.push(image);
        if (i === 0) {
          image.onload = function (event) {
            resizeCanvasToImageRatio(event.target.width, event.target.height);
            drawImage(event.target);
            prevButton.onclick = prevSlide;
            nextButton.onclick = nextSlide;
						callback(null);
          };
					image.onerror = function (event) {
						callback(new Error('First image could not be loaded'));
					}
        }
        currentImageIndex += 1;
      }
    }
  }

  $.fn.createPresentation = function(config, callback) {
    var self = this;
    var errors = false;
    context = self[0].getContext('2d');
    if (!config.presentationLenght) {
      callback(new Error('No presentation length provided'));
      errors = true;
    } else if (!config.prevButton || !config.nextButton) {
      callback(new Error('No elements for navigation buttons are provided'));
      errors = true;
    } else if (!context) {
      callback(new Error('Could not get canvas context'));
      errors = true;
    } else if (!config.directory && !config.firstImagePath) {
      callback(new Error('Image source wasn\'t provided'));
      errors = true;
    } else if (config.firstImagePath && !/.svg$/.test(config.firstImagePath)) {
      callback(new Error('Slides should be .svg images'));
      errors = true;
    }
    if (!errors) {
      if (config.firstImagePath) {
        var pathConfig = config.firstImagePath.match(/(.+\-)(\d+)\.svg$/);
        path = pathConfig[1];
        zeroPadding = pathConfig[2].match(/(0*)\d$/)[1];
        currentImageIndex = parseInt(pathConfig[2]);
      } else if (config.directory) {
        path = config.directory;
        if (!/\/$/.test(config.directory)) {
          path += '/';
        }
        if (config.slideTemplate) {
          var template = config.slideTemplate.match(/(.+)(\d+)\.svg$/);
          path += template[1];
          zeroPadding = template[2].match(/(0*)\d$/)[1];
          currentImageIndex = parseInt(template[2]);
        } else {
          path += 'slide-';
          zeroPadding = '';
          currentImageIndex = 1;
        }
      }
      presentationLength = config.presentationLenght;
      initPresentation(config.prevButton, config.nextButton, callback);
    }
    return this;
  };

  $.fn.rerenderPresentation = function() {
    resizeCanvasToImageRatio(images[currentSlide].width, images[currentSlide].height);
    drawImage(images[currentSlide]);
    return this;
  }

}(jQuery));
